<?php

namespace IMATHUZH\Qfq\Core\Renderer;

use IMATHUZH\Qfq\Core\Database\DatabaseManager;
use IMATHUZH\Qfq\Core\Form\Form;
use IMATHUZH\Qfq\Core\Form\FormElement\AbstractFormElement;
use IMATHUZH\Qfq\Core\Helper\HelperFormElement;
use IMATHUZH\Qfq\Core\Helper\Support;
use IMATHUZH\Qfq\Core\Renderer\Element\ButtonRenderer;
use IMATHUZH\Qfq\Core\Renderer\Element\ElementRenderer;
use IMATHUZH\Qfq\Core\Renderer\Element\SpinnerRenderer;
use IMATHUZH\Qfq\Core\Renderer\FormElement\Base\CheckboxRenderer;
use IMATHUZH\Qfq\Core\Renderer\FormElement\Base\DatetimeRenderer;
use IMATHUZH\Qfq\Core\Renderer\FormElement\Base\EditorRenderer;
use IMATHUZH\Qfq\Core\Renderer\FormElement\Base\FieldsetRenderer;
use IMATHUZH\Qfq\Core\Renderer\FormElement\Base\ImageCutRenderer;
use IMATHUZH\Qfq\Core\Renderer\FormElement\Base\InputRenderer;
use IMATHUZH\Qfq\Core\Renderer\FormElement\Base\NoteRenderer;
use IMATHUZH\Qfq\Core\Renderer\FormElement\Base\PillRenderer;
use IMATHUZH\Qfq\Core\Renderer\FormElement\Base\RadioButtonRenderer;
use IMATHUZH\Qfq\Core\Renderer\FormElement\Base\SelectRenderer;
use IMATHUZH\Qfq\Core\Renderer\FormElement\Base\SubrecordRenderer;
use IMATHUZH\Qfq\Core\Renderer\FormElement\Base\UploadRenderer;
use IMATHUZH\Qfq\Core\Renderer\FormElement\FormElementRenderer;
use IMATHUZH\Qfq\Core\Store\Store;


class BaseRenderer
{
    // Instances / Managers
    protected ?Store $store = null;
    protected ?DatabaseManager  $databaseManager = null;



    // ElementRenderers
    protected ?ElementRenderer $buttonRenderer = null;
    protected ?ElementRenderer $spinnerRenderer = null;

    // Container FormElementRenderers
    protected ?FormElementRenderer $pillRenderer = null;
    protected ?FormElementRenderer $fieldsetRenderer = null;

    // Native FormElementRenderers
    protected ?FormElementRenderer $noteRenderer = null;
    protected ?FormElementRenderer $inputRenderer = null;
    protected ?FormElementRenderer $checkboxRenderer = null;
    protected ?FormElementRenderer $radioRenderer = null;
    protected ?FormElementRenderer $selectRenderer = null;
    protected ?FormElementRenderer $subrecordRenderer = null;
    protected ?FormElementRenderer $editorRenderer = null;
    protected ?FormElementRenderer $datetimeRenderer = null;
    protected ?FormElementRenderer $uploadRenderer = null;
    protected ?FormElementRenderer $imageCutRenderer = null;


    //<editor-fold desc="Singleton">
    protected static ?BaseRenderer $instance = null;

    /**
     * @return BaseRenderer
     * @throws \CodeException
     * @throws \DbException
     * @throws \UserFormException
     * @throws \UserReportException
     */
    public static function getInstance(): BaseRenderer
    {
        if(self::$instance == null){
            $specifiedRenderer = "IMATHUZH\Qfq\Core\Renderer\\" .  Store::getVar("renderer", STORE_SYSTEM);
            if(class_exists($specifiedRenderer)){
                self::$instance = new $specifiedRenderer;
            }
            else{
                self::$instance = new BaseRenderer();
            }
        }
        return self::$instance;
    }
    //</editor-fold>

    public function __construct()
    {
        $this->store = Store::getInstance();
        $this->databaseManager = DatabaseManager::getInstance();

        $this->pillRenderer = new PillRenderer();
        $this->fieldsetRenderer = new FieldsetRenderer();

        $this->buttonRenderer = new ButtonRenderer();
        $this->spinnerRenderer = new SpinnerRenderer();

        $this->inputRenderer = new InputRenderer();
        $this->checkboxRenderer = new CheckboxRenderer();
        $this->radioRenderer = new RadioButtonRenderer();
        $this->subrecordRenderer = new SubrecordRenderer();
        $this->selectRenderer = new SelectRenderer();
        $this->noteRenderer = new NoteRenderer();
        $this->editorRenderer = new EditorRenderer();
        $this->datetimeRenderer = new DatetimeRenderer();
        $this->imageCutRenderer = new ImageCutRenderer();
        $this->uploadRenderer = new UploadRenderer();

    }



    public function renderForm(Form $form, string $renderMode = RENDER_MODE_HTML) : string{
        $output = $this->renderFormHead($form, $renderMode);
        for($i = 0; $i < count($form->formElements) ; $i++){
            $output .= $this->renderFormElement($form->formElements[$i]);
        }
        $output .= $this->renderFormTail($form, $renderMode);
        return $output;
    }

    protected function renderFormHead(Form $form, string $renderMode = RENDER_MODE_HTML) : string{
        $output = "";
        return $output;
    }

    protected function renderFormTail(Form $form, string $renderMode = RENDER_MODE_HTML) : string{
        $output = "";
        return $output;
    }

    public function renderFormElement(AbstractFormElement $formElement, string $renderMode = RENDER_MODE_HTML) : string{
        $output = "";

        switch (get_class($formElement)){
            case "IMATHUZH\Qfq\Core\Form\FormElement\NoteFormElement":
                $output = $this->noteRenderer->renderFormElement($formElement);
                break;
            case "IMATHUZH\Qfq\Core\Form\FormElement\InputFormElement":
                $output = $this->inputRenderer->renderFormElement($formElement);
                break;
            case "IMATHUZH\Qfq\Core\Form\FormElement\EditorFormElement":
                $output = $this->editorRenderer->renderFormElement($formElement);
                break;
            case "IMATHUZH\Qfq\Core\Form\FormElement\DatetimeFormElement":
                $output = $this->datetimeRenderer->renderFormElement($formElement);
                break;
            case "IMATHUZH\Qfq\Core\Form\FormElement\CheckboxFormElement":
                $output = $this->checkboxRenderer->renderFormElement($formElement);
                break;
            case "IMATHUZH\Qfq\Core\Form\FormElement\RadioButtonFormElement":
                $output = $this->radioRenderer->renderFormElement($formElement);
                break;
            case "IMATHUZH\Qfq\Core\Form\FormElement\SelectFormElement":
                $output = $this->selectRenderer->renderFormElement($formElement);
                break;
            case "IMATHUZH\Qfq\Core\Form\FormElement\ExtraFormElement":
                $output = ""; // Extra has no output, that's the whole point lol
                break;
            case "IMATHUZH\Qfq\Core\Form\FormElement\UploadFormElement":
                $output = $this->uploadRenderer->renderFormElement($formElement);
                break;
            case "IMATHUZH\Qfq\Core\Form\FormElement\ImageCutFormElement":
                $output = $this->imageCutRenderer->renderFormElement($formElement);
                break;
            case "IMATHUZH\Qfq\Core\Form\FormElement\SubrecordFormElement":
                $output = $this->subrecordRenderer->renderFormElement($formElement);
                break;

            case "IMATHUZH\Qfq\Core\Form\FormElement\PillFormElement":
                $output = $this->pillRenderer->renderFormElement($formElement);
                break;
            case "IMATHUZH\Qfq\Core\Form\FormElement\FieldsetFormElement":
                $output = $this->fieldsetRenderer->renderFormElement($formElement);
                break;

            default:
                $output = '<p>No Renderer Class for found for ' . get_class($formElement) . '</p>';
                break;
        }
        return $output;
    }

    public function renderElement(array $elementSpec, string $renderMode = RENDER_MODE_HTML){

    }

    /**
     * Create HTML Input vars to detect bot automatic filling of forms.
     *
     * @return string
     * @throws \CodeException
     * @throws \UserFormException
     */
    protected function renderHoneypotElements() : string{
        $html = '';

        $vars = $this->store->getVar(SYSTEM_SECURITY_VARS_HONEYPOT, STORE_SYSTEM);

        // Iterate over all fake vars
        $arr = explode(',', $vars);
        foreach ($arr as $name) {
            $name = trim($name);
            if ($name === '') {
                continue;
            }
            $html .= "<input name='$name' type='hidden' value='' readonly>";
        }

        return $html;
    }

    /**
     * Build MD5 from the current record. Return HTML Input element.
     *
     * @return string
     * @throws \CodeException
     * @throws \DbException
     * @throws \UserFormException
     */
    protected function renderRecordHashMd5Input(Form $form) {
        $md5 = $form->getRecordHashMd5();
        return "<input id='" . DIRTY_RECORD_HASH_MD5 . "' name='" . DIRTY_RECORD_HASH_MD5 . "' type='hidden' value='$md5'>";
    }

    /**
     * Copy a subset of current STORE_TYPO3 variables to SIP. Set a hidden form field to submit the assigned SIP to
     * save/update.
     *
     * @return string
     * @throws \CodeException
     * @throws \UserFormException
     */
    protected function renderHiddenT3Sip() {
        $t3VarsSip = $this->store->copyT3VarsToSip();
        return HelperFormElement::buildNativeHidden(CLIENT_TYPO3VARS, $t3VarsSip);
    }


    protected function renderHiddenSip() {

        $sipArray = $this->store->getStore(STORE_SIP);

        // do not include system vars
        unset($sipArray[SIP_SIP]);
        unset($sipArray[SIP_URLPARAM]);

        $queryString = Support::arrayToQueryString($sipArray);
        $sip = $this->store->getSipInstance();
        $sipValue = $sip->queryStringToSip($queryString, RETURN_SIP);

        $wrapSetupClass = $this->wrap[WRAP_SETUP_ELEMENT][WRAP_SETUP_CLASS] ?? '';
        //$json[] = $this->getFormElementForJson(CLIENT_SIP, $sipValue, [FE_MODE => FE_MODE_SHOW], $wrapSetupClass);

        return HelperFormElement::buildNativeHidden(CLIENT_SIP, $sipValue);
    }
}